///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hbin;

import 'HVal.dart';
import 'string_utils.dart';

/// HBin models a binary file with a MIME type.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HBin extends HVal {
  /// Construct for MIME type
  HBin(this.mime) {
    if (mime.isEmpty || !mime.contains('/')) {
      throw new FormatException("Invalid mime val: \"$mime\"", mime);
    }

    _verifyMime(mime);
  }

  /// MIME type for binary file
  final String mime;

  /// Hash code is based on mime field
  @override
  int get hashCode => mime.hashCode;

  /// Equals is based on mime field
  @override
  bool operator ==(Object that) => that is HBin && this.mime == that.mime;

  /// Encodes to JSON string value
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("b:");
    s.write(mime);
    return s.toString();
  }

  /// Encodes as "Bin(<mime>)"
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    s..write("Bin(")..write('"')..write(mime)..write('"')..write(')');
    return s.toString();
  }

  static void _verifyMime(String _mime) {
    _mime.codeUnits.forEach((int c) {
      if (c > 127 || c == CLOSE_PARENTHESE_CHAR) {
        throw new FormatException(
            "Invalid mime, char='${new String.fromCharCode(c)}'", _mime);
      }
    });
  }
}
