///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

part of haystack.hgrid;

/// HHisItem is used to model a timestamp/value pair
///
/// See [Project Haystack](http://project-haystack.org/doc/Ops#hisRead)
class HHisItem extends HDict {
  /// Map [HGrid] to [HHisItem].  Grid must have ts and val columns.
  static List<HHisItem> gridToItems(HGrid grid) {
    HCol ts = grid.colByName("ts")!;
    HCol val = grid.colByName("val")!;

    final items = grid.map<HHisItem>((row) {
      return HHisItem(
          row.getByCol(ts, checked: true) as HDateTime,
          row.getByCol(val, checked: false)!,
      );
    }).toList();

    return items;
  }

  /// Construct from timestamp, value
  factory HHisItem(HDateTime ts, HVal val) {
    return new HHisItem._(ts, val);
  }

  HHisItem._(this.ts, this.val) : super._();

  /// Timestamp of history sample
  final HDateTime ts;

  /// Value of history sample
  final HVal val;

  @override
  int get size => 2;

  @override
  HVal? get(String name, {bool checked = true}) {
    if (name == "ts") return ts;
    if (name == "val") return val;
    if (!checked) return null;
    throw new UnknownNameError(name);
  }

  @override
  HVal? operator [](Object? key) => get(key.toString(), checked: false);

  @override
  Iterable<String> get keys => ["ts", "val"];
}