///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hstr;

import 'HVal.dart';
import 'string_utils.dart';

/// HStr wraps a dart.core.String as a tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HStr extends HVal {
  /// Construct from [String] value.
  factory HStr(String val) {
    if (val.isEmpty) return _EMPTY;

    return new HStr._(val);
  }

  /// Singleton value for empty string "".
  static final HStr _EMPTY = HStr._("");

  HStr._(this.val);

  /// String value.
  final String val;

  /// Hash code is same as [String.hashCode].
  @override
  int get hashCode => val.hashCode;

  /// Equals is based on [String].
  @override
  bool operator ==(Object that) => that is HStr && this.val == that.val;

  /// Returns value string.
  @override
  String toString() => val;

  /// Encodes as "s:" if string contains a colon.
  @override
  String toJson() => !val.contains(':') ? val : "s:$val";

  /// Encodes using double quotes and back slash escapes.
  @override
  String toZinc() {
    var s = new StringBuffer();
    HStr.encodeForZinc(s, val);
    return s.toString();
  }

  /// Encodes using double quotes and back slash escapes.
  static String toCode(String val) {
    StringBuffer s = new StringBuffer();
    encodeForZinc(s, val);
    return s.toString();
  }

  /// Encodes using double quotes and back slash escapes.
  static void encodeForZinc(StringBuffer s, String val) {
    s.write('"');
    val.codeUnits.forEach((int c) {
      if (c < SPACE_CHAR || c == DOUBLE_QUOTE_CHAR || c == BACKSLASH_CHAR) {
        s.write('\\');
        switch (c) {
          case NL_CHAR:
            s.write('n');
            break;
          case CR_CHAR:
            s.write('r');
            break;
          case TAB_CHAR:
            s.write('t');
            break;
          case DOUBLE_QUOTE_CHAR:
            s.write('"');
            break;
          case BACKSLASH_CHAR:
            s.write('\\');
            break;
          default:
            s..write('u')..write('0')..write('0');
            if (c <= 0xf) {s.write('0');}
            s.write(c.toRadixString(16));
        }
      } else {
        s.writeCharCode(c);
      }
    });
    s.write('"');
  }
}
