///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.htime_zone;

/// HTimeZone handles the mapping between Haystack timezone
/// names and Java timezones. (No, in Dart there isn't timezones)
///
/// See [Project Haystack](http://project-haystack.org/doc/TimeZones)
class HTimeZone {
  /// Haystack timezone name.
  final String name;

  /// in seconds
  final int offset;

  factory HTimeZone.local() {
    DateTime now = new DateTime.now();
    return new HTimeZone(now.timeZoneName, now.timeZoneOffset.inSeconds);
  }

  HTimeZone(this.name, [this.offset = 0]);

  static final HTimeZone UTC = new HTimeZone('UTC', 0);

  static final HTimeZone DEFAULT = new HTimeZone.local();

  @override
  int get hashCode => offset.hashCode;

  @override
  bool operator ==(Object that) => that is HTimeZone && that.offset == offset;

  /// Returns Haystack timezone name.
  @override
  String toString() => name;
}
