///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.huri;

import 'HVal.dart';
import 'string_utils.dart';

/// HUri models a URI as a string.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HUri extends HVal {
  /// Singleton value for empty URI.
  static final HUri EMPTY = new HUri._("");

  /// String value of URI.
  final String val;

  /// Hash code is based on string value.
  @override
  int get hashCode => val.hashCode;

  /// Factory constructor from string value.
  factory HUri(String val) {
    if (val.isEmpty) return EMPTY;
    return new HUri._(val);
  }

  /// Private constructor.
  HUri._(this.val);

  /// Equals is based on string value.
  @override
  bool operator ==(Object that) => that is HUri && this.val == that.val;

  /// Returns value string..
  @override
  String toString() => val;

  /// Encodes as "u:<val>".
  @override
  String toJson() => "u:$val";

  /// Encodes using "`" back ticks.
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    s.write('`');

    val.codeUnits.forEach((int c) {
      if (c < SPACE_CHAR) throw new FormatException("Invalid URI char: '${new String.fromCharCode(c)}'", val);
      if (c == BACKQUOTE_CHAR) {
        s.write('\\');
      }
      s.writeCharCode(c);
    });

    s.writeCharCode(BACKQUOTE_CHAR);
    return s.toString();
  }
}
