///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.token;

class HaystackToken {
  static const HaystackToken eof = HaystackToken("eof");

  static const HaystackToken id = HaystackToken("identifier");
  static const HaystackToken num = HaystackToken("Number", true);
  static const HaystackToken str = HaystackToken("Str", true);
  static const HaystackToken ref = HaystackToken("Ref", true);
  static const HaystackToken uri = HaystackToken("Uri", true);
  static const HaystackToken date = HaystackToken("Date", true);
  static const HaystackToken time = HaystackToken("Time", true);
  static const HaystackToken dateTime = HaystackToken("DateTime", true);

  static const HaystackToken colon = HaystackToken(":");
  static const HaystackToken comma = HaystackToken(",");
  static const HaystackToken semicolon = HaystackToken(";");
  static const HaystackToken minus = HaystackToken("-");
  static const HaystackToken eq = HaystackToken("==");
  static const HaystackToken notEq = HaystackToken("!=");
  static const HaystackToken lt = HaystackToken("<");
  static const HaystackToken lt2 = HaystackToken("<<");
  static const HaystackToken ltEq = HaystackToken("<=");
  static const HaystackToken gt = HaystackToken(">");
  static const HaystackToken gt2 = HaystackToken(">>");
  static const HaystackToken gtEq = HaystackToken(">=");
  static const HaystackToken lbracket = HaystackToken("[");
  static const HaystackToken rbracket = HaystackToken("]");
  static const HaystackToken lbrace = HaystackToken("{");
  static const HaystackToken rbrace = HaystackToken("}");
  static const HaystackToken lparen = HaystackToken("(");
  static const HaystackToken rparen = HaystackToken(")");
  static const HaystackToken arrow = HaystackToken("->");
  static const HaystackToken slash = HaystackToken("/");
  static const HaystackToken assign = HaystackToken("=");
  static const HaystackToken bang = HaystackToken("!");
  static const HaystackToken nl = HaystackToken("newline");

  static const HaystackToken space = HaystackToken("space");
  static const HaystackToken tab = HaystackToken("tab"); // including non-breaking space (0xa0)
  static const HaystackToken trioEntitiesDelimiter = HaystackToken("trioEntitiesDelimiter");


  const HaystackToken(this.symbol, [bool? literal])
      : this.literal = literal ?? false;

  @override
  bool operator ==(Object that) {
    return that is HaystackToken &&
        literal == that.literal &&
        symbol == that.symbol;
  }

  @override
  int get hashCode {
    int result = symbol.hashCode;
    result = 31 * result + (literal ? 1 : 0);
    return result;
  }

  @override
  String toString() => symbol;

  final String symbol;
  final bool literal;
}
