library haystack.htrioreader;


import 'package:haystack/haystack.dart';
import 'package:haystack/src/HGrid.dart';
import 'package:haystack/src/HGridBuilder.dart';
import 'package:haystack/src/HVal.dart';
import 'package:haystack/src/io/HGridReader.dart';
import 'package:haystack/src/io/TrioTokenizer.dart';
import 'package:haystack/src/io/trio_token.dart';


/// Used to read grids using the Trio format.
///
/// See [Project Haystack](https://project-haystack.org/doc/Trio)
class MyHTrioReader implements HGridReader {

  // todo: null case for "s"
  MyHTrioReader(String s)
      : this._tokenizer = TrioTokenizer(s.runes)
  {
    _init();
  }

  void _init() {
    _consume();
    _consume();
  }

  @override
  HGrid readGrid() {
    List<Map<String, HVal>> entities = <Map<String, HVal>>[];

    int whileCounter = 0;
    while (_cur != TrioToken.eof) {
      print("----------------------------");
      print("Parsing record: $whileCounter");
      Map<String, HVal> record = _parseRecord();
      entities.add(record);
      ++whileCounter;
    }

    HGridBuilder gridBuilder = HGridBuilder();
    List<String> colNames = entities
        .expand((Map<String, HVal> e) => e.keys)
        .toSet()
        .toList();

    colNames.forEach((String colName) => gridBuilder.addCol(colName));

    entities.forEach((Map<String, HVal> e) {
      List<HVal?> row = <HVal?>[];

      // todo: do we need fill cell with HNull if there is no value of null is ok?
      colNames.forEach((String col) => row.add(e[col]));

      gridBuilder.addRow(row);
    });


    HGrid result = gridBuilder.toGrid();
    return result;

//    bool nested = _cur == HaystackToken.lt2;
//
//    if (nested) {
//      _consume(HaystackToken.lt2);
//      if (_cur == HaystackToken.nl) {
//        _consume(HaystackToken.nl);
//      }
//    }
//
//    // ver:"3.0"
//    if (_cur != HaystackToken.id || "ver" != _curVal) {
//      throw _err("Expecting grid 'ver' identifier, not " + _curToStr());
//    }
//    _consume();
//    _consume(HaystackToken.colon);
//    this._version = _checkVersion(_consumeStr());
//
//    // grid meta
//    HGridBuilder gb = new HGridBuilder();
//
//    if (_cur == HaystackToken.id) {
//      gb.meta.addDict(_parseDict());
//    }
//    _consume(HaystackToken.nl);
//
//    // column definitions
//    int numCols = 0;
//
//    while (_cur == HaystackToken.id) {
//      ++numCols;
//      String name = _consumeTagName();
//      HDict colMeta = HDict.EMPTY;
//
//      if (_cur == HaystackToken.id) {
//        colMeta = _parseDict();
//      }
//      gb.addCol(name).addDict(colMeta);
//
//      if (_cur != HaystackToken.comma) break;
//      _consume(HaystackToken.comma);
//    }
//
//    if (numCols == 0) throw _err("No columns defined");
//    _consume(HaystackToken.nl);
//
//    // grid rows
//    while (true) {
//      if (_cur == HaystackToken.nl) break;
//      if (_cur == HaystackToken.eof) break;
//      if (nested && _cur == HaystackToken.gt2) break;
//
//      // read cells
//      List<HVal> cells = new List(numCols);
//
//      for (int i = 0; i < numCols; ++i) {
//        if (_cur == HaystackToken.comma ||
//            _cur == HaystackToken.nl ||
//            _cur == HaystackToken.eof) {
//          cells[i] = null;
//        } else {
//          cells[i] = _parseVal();
//        }
//        if (i + 1 < numCols) {
//          _consume(HaystackToken.comma);
//        }
//      }
//
//      gb.addRow(cells);
//
//      // newline or end
//      if (nested && _cur == HaystackToken.gt2) break;
//      if (_cur == HaystackToken.eof) break;
//      _consume(HaystackToken.nl);
//    }
//
//    if (_cur == HaystackToken.nl) {
//      _consume(HaystackToken.nl);
//    }
//    if (nested) {
//      _consume(HaystackToken.gt2);
//    }
//    return gb.toGrid();
  }

  Map<String, HVal> _parseRecord() {
    print("_parseRecord starts");
    Map<String, HVal> result = Map<String, HVal>();

    int whileCounter = 0;
    while(!_isEndOfRecord()) {
      _skipEmptyLines();
      if (_isEndOfRecord()) break; // empty record case.
//      print("cur: $_cur, val: ${_cur.value}");
      print("parsing tagValue: $whileCounter ($_cur)");
      MapEntry<String, HVal> tagValuePair = _parseTagValuePair();
      result.addEntries([tagValuePair]);
      ++whileCounter;

      if (_cur == TrioToken.nl) _skipEmptyLines();
    }

    _consume(); //skip delimiter
    print("_parseRecord ends. Result is: $result");
    return result;
  }

  /// Returns if current tokenizer define end of record.
  /// Record ends in 2 cases:
  /// - delimiter
  /// - eof
  bool _isEndOfRecord() {
    return _cur == TrioToken.trioEntitiesDelimiter || _cur == TrioToken.eof;
  }

  void _skipEmptyLines() {
    while(_isNewLine() || _isSpace()) {
      _skipNewLines();
      _skipSpaces();
    }
  }

  MapEntry<String, HVal> _parseTagValuePair() {
    String tagName;
    HVal val;

    tagName = _getTagName();
    if (_cur != TrioToken.trioEntitiesDelimiter) _consume(); // skip handled tag name
    print("got tag name: $tagName");
    val = _getTagVal();
    if (_cur != TrioToken.trioEntitiesDelimiter) _consume(); //skip handled value

    print("tag value pair is: $tagName: $val(${val.runtimeType})");
    return MapEntry(tagName, val);
  }

  String _getTagName() {
    _skipSpaces();

    //todo: tokenizer returns HaystackToken.id even if string doesn't match naming rules for tags.
    //todo: In skyspark we are only allowed to have tag names started with lower case letter
    //todo: but tokenizer parses as id even a strings started with upper case.
    //todo: do we need add some validation here or skip because Trio format itself doesn't require it.
    if (_cur!.type == TokenType.identifier) {
      return _cur!.value as String;
    }
    else throw ParseError("Record must start with the identifier (tag name)");
  }

  HVal _getTagVal() {
    _skipSpaces();
    HVal result;

    bool nextLineWithoutVal = _cur == TrioToken.nl || _cur == TrioToken.eof || _cur == TrioToken.trioEntitiesDelimiter;
    bool isColon = _cur == TrioToken.colon;

    if (nextLineWithoutVal) {
      result = HMarker.VAL;
    } else if (isColon) {
      _consume(); // skip colon
      _skipSpaces();

      bool isMultilineStr = _cur == TrioToken.nl && _peek == TrioToken.space;
      bool isEmptyVal = (!isMultilineStr && _cur == TrioToken.nl ) || _cur == TrioToken.trioEntitiesDelimiter || _cur == TrioToken.eof;
      print("getting tag value...");
      print("isMultilineStr: ${isMultilineStr}, isEmptyVal: ${isEmptyVal}");

      // if there is no value after ":" it means empty string
      if (isEmptyVal) result = HStr("");
      else result = _getNotEmptyTagValue();

    } else {
      throw ParseError("Can't parse tag value");
    }

    return result;
  }

  /// Handle case when value exists and it is not empty (there is data after ":").
  ///
  /// For common handling tag value use [_getTagVal].
  HVal _getNotEmptyTagValue() {
    bool simpleVal = _peek == TrioToken.eof || _peek == TrioToken.trioEntitiesDelimiter || _peek == TrioToken.nl;
    HVal result;
    if (simpleVal) result = _cur!.value as HVal;
    else {
      bool isMultilineStr = _cur == TrioToken.nl && _peek == TrioToken.space;
      result = isMultilineStr
        ? _getMultiLineStr()
        : _getUnquotedStr();
    }
//    1. Проверить будущий токен.
//    1а) Если он eof или разделитель рекордов, то только текущий токен представляет собой value. Парсим текуший токен и возвращаем value.
//    1б) Если будущий токен может быть частью value (например пробел или новая строка при многострочной строке), то собираем value с помощью consume до тех по

    return result;
  }

  HVal _getMultiLineStr() {
    List<String> lines = <String>[];

    bool hasOneMoreLine = _cur == TrioToken.nl && _peek == TrioToken.space;

    while(hasOneMoreLine) {
      _consume(); // skip new line
      _skipSpaces(); //skip intent which defines the multi-line string

      String curStr = _getUnquotedStr().toString().trim();
      lines.add(curStr);
      hasOneMoreLine = _cur == TrioToken.nl && _peek == TrioToken.space;
    }

    String resultStr = lines.join(" ");
    return HStr(resultStr);
  }

  HVal _getUnquotedStr() {
    String resultStr = "";

    bool isEndOfStr = _cur == TrioToken.nl || _cur == TrioToken.eof || _cur == TrioToken.trioEntitiesDelimiter;
    while (!isEndOfStr) {
      // Skip tabs (it is done in Skyspark ioReadTrio).
      if (_cur != TrioToken.tab) {
        String curStr = _cur!.value.toString();

        // If token was parsed as a string it means it was quoted.
        // But multiline is unquoted so we need to return quotes back as a part of string.
        if (_cur!.type == TokenType.str) curStr = _quoteStr(curStr);

        resultStr += curStr;
      }

      _consume();
      isEndOfStr = _cur == TrioToken.nl || _cur == TrioToken.eof || _cur == TrioToken.trioEntitiesDelimiter;
    }

    return HStr(resultStr);
  }

  String _quoteStr(String str) => "\"$str\"";

  /// Skips non-meaningful spaces.
  void _skipSpaces() {
    while(_isSpace()) {
      _consume();
    }
  }

  /// Returns if current token is space.
  /// Treat space, tab, non-breaking space as whitespace.
  bool _isSpace() => _cur == TrioToken.space;

  /// Returns if current token is new line.
  bool _isNewLine() => _cur ==  TrioToken.nl;

  /// Skips new lines.
  void _skipNewLines() {
    while(_isNewLine()) {
        _consume();
    }
  }

//  void _consume([HaystackToken expected]) {
  void _consume() {
//    if (expected != null) {
//      _verify(expected);
//    }

    _cur = _peek;
    _tokenizer.next();
    _peek = _tokenizer.tok;

  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  final TrioTokenizer _tokenizer;
  TrioToken? _cur;
  TrioToken? _peek;
}