class TrioToken {
  /// Type of the token.
  final TokenType type;

  /// Name of the token.
  ///
  /// If [type] is [TokenType.syntax] it is syntax symbol/s itself.
  /// If [type] is [TokenType.data] it is description based on [value].
  final String name;

  /// Value of the token if it is not syntax token.
  ///
  /// If token has type [TrioToken.syntax] value usually literal of the syntax sign/s.
  final Object value;


//
//  static const TrioToken id = TrioToken._("identifier");
//  static const TrioToken num = TrioToken._("Number", true);
//  static const TrioToken str = TrioToken._("Str", true);
//  static const TrioToken ref = TrioToken._("Ref", true);
//  static const TrioToken uri = TrioToken._("Uri", true);
//  static const TrioToken date = TrioToken._("Date", true);
//  static const TrioToken time = TrioToken._("Time", true);
//  static const TrioToken dateTime = TrioToken._("DateTime", true);

  static const TrioToken colon = TrioToken.syntax(":");
  static const TrioToken comma = TrioToken.syntax(",");
  static const TrioToken semicolon = TrioToken.syntax(";");
  static const TrioToken minus = TrioToken.syntax("-");
  static const TrioToken eq = TrioToken.syntax("==");
  static const TrioToken notEq = TrioToken.syntax("!=");
  static const TrioToken lt = TrioToken.syntax("<");
  static const TrioToken lt2 = TrioToken.syntax("<<");
  static const TrioToken ltEq = TrioToken.syntax("<=");
  static const TrioToken gt = TrioToken.syntax(">");
  static const TrioToken gt2 = TrioToken.syntax(">>");
  static const TrioToken gtEq = TrioToken.syntax(">=");
  static const TrioToken lbracket = TrioToken.syntax("[");
  static const TrioToken rbracket = TrioToken.syntax("]");
  static const TrioToken lbrace = TrioToken.syntax("{");
  static const TrioToken rbrace = TrioToken.syntax("}");
  static const TrioToken lparen = TrioToken.syntax("(");
  static const TrioToken rparen = TrioToken.syntax(")");
  static const TrioToken arrow = TrioToken.syntax("->");
  static const TrioToken slash = TrioToken.syntax("/");
  static const TrioToken assign = TrioToken.syntax("=");
  static const TrioToken bang = TrioToken.syntax("!");
  static const TrioToken nl = TrioToken.syntax("newline");

  static const TrioToken space = TrioToken.syntax("space", value: " "); // including non-breaking space (0xa0)
  static const TrioToken tab = TrioToken.syntax("tab");
  static const TrioToken trioEntitiesDelimiter = TrioToken.syntax("trioEntitiesDelimiter");
  static const TrioToken eof = TrioToken.syntax("eof");


  const TrioToken.data(this.type, this.name, this.value);

  const TrioToken.syntax(this.name, {String? value})
      : type = TokenType.syntax,
        value = value ?? name;

  @override
  bool operator ==(Object that) {
    return that is TrioToken &&
        name == that.name &&
        value == that.value;
  }

  @override
  int get hashCode {
    int result = name.hashCode;
    int valueHashCode = value.hashCode;
    result = 31 * result + valueHashCode;
    return result;
  }

  @override
  String toString() => name;
}

enum TokenType {
  syntax,
  identifier, //tag name
  num,
  ref,
  str,
  uri,
  date,
  time,
  dateTime
}
