///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.src.utils.simplify;

import 'package:haystack/haystack.dart';


/// Makes HVals more simpler, for commit call.
///
/// Makes ids ([HRef]) relative and removes dis part.
T simplifyVal<T extends HVal>(T val) {
  if (val is HRef) {
    final newId = val.val.substring(val.val.lastIndexOf(':') + 1);
    return new HRef(newId) as T;
  }

  if (val is HGrid) {
    var rows = val.map((row) => simplifyVal<HRow>(row)).toList();
    return HGridBuilder.dictsToGrid(val.meta, rows) as T;
  }

  if (val is HRow) {
    var newValues = val.values.map((v) {
      if (v != null) {
        return simplifyVal<HVal>(v);
      } else {
        return v;
      }
    }).toList();

    return new HRow(val.grid, newValues) as T;
  }

  if (val is HDict && val is! HHisItem) {
    var newMap = <String, HVal?>{};

    val.keys.forEach((key) {
      final value = val[key];
      if (value != null) {
        newMap[key] = simplifyVal<HVal>(value);
      } else {
        newMap[key] = value;
      }
    });

    return new HDict(newMap) as T;
  }

  return val;
}

Iterable<T> simplifyList<T extends HVal>(Iterable<T> values) {
  return values.map<T>((val) => simplifyVal<T>(val));
}