import 'package:haystack/haystack.dart';
import 'package:haystack/src/io/HTrioWriter.dart';
import 'package:haystack/src/string_utils.dart';
import 'package:test/test.dart';

void main() {
  group("HTrioWriter tests.", () {

    test("Write empty grid correct.", () {
      HGrid testGrid = HGrid.EMPTY;
      StringBuffer stringBuffer = StringBuffer();
      HTrioWriter writer = HTrioWriter(stringBuffer);
      writer.writeGrid(testGrid);

      String result = stringBuffer.toString();
      bool isEmpty = result.isEmpty;

      expect(isEmpty, true);
    });

    test("Write grid with only empty records correct.", () {
      HDict emptyRecord = HDict({});
      HDict meta = HDict({});
      HGrid testGrid = HGridBuilder.dictsToGrid(meta, [emptyRecord, emptyRecord, emptyRecord]);
      StringBuffer stringBuffer = StringBuffer();
      HTrioWriter writer = HTrioWriter(stringBuffer);
      writer.writeGrid(testGrid);

      String result = stringBuffer.toString();
      print(result);

      int linesCount = result.split("\n").length;
      expect(linesCount, 3); //last line is empty

      int delimitersCount = result.split("\n-").length - 1;
      bool isFirstDelimiter = result.codeUnitAt(0) == MINUS_CHAR;
      if (isFirstDelimiter) delimitersCount++;
      expect(delimitersCount, 2, reason: "Must be 2 delimiters for 3 records");
    });

    test("Write grid with one simple record correct.", () {
      Map<String, HVal> recordTags = {
        "numericTag": HNum(25),
        "stringTag": HStr("test string"),
        "refTag": HRef("thisIsTheRef"),
        "boolTag": HBool.FALSE
      };

      HDict record = HDict(recordTags);
      HDict meta = HDict({});
      HGrid testGrid = HGridBuilder.dictsToGrid(meta, [record]);
      StringBuffer stringBuffer = StringBuffer();
      HTrioWriter writer = HTrioWriter(stringBuffer);
      writer.writeGrid(testGrid);

      String result = stringBuffer.toString();
      int linesCount = result.split("\n").length;
      bool containsDelimiter = result.contains("\n-");

      expect(linesCount, 5); //last line is empty
      expect(containsDelimiter, false);
    });

    test("Not empty records are destinguished by delimiter correct.", () {
      Map<String, HVal> recordTags = {
        "numericTag": HNum(25),
        "stringTag": HStr("test string"),
        "refTag": HRef("thisIsTheRef"),
        "boolTag": HBool.FALSE
      };

      HDict record1 = HDict(recordTags);
      HDict record2 = HDict(recordTags);
      HDict record3 = HDict(recordTags);
      HDict meta = HDict({});
      HGrid testGrid = HGridBuilder.dictsToGrid(meta, [record1, record2, record3]);
      StringBuffer stringBuffer = StringBuffer();
      HTrioWriter writer = HTrioWriter(stringBuffer);
      writer.writeGrid(testGrid);

      String result = stringBuffer.toString();
      int delimitersCount = result.split("\n-").length - 1;
      expect(delimitersCount, 2);
    });

    test("Not empty and empty records are destinguished by delimiter correct.", () {
      Map<String, HVal> recordTags = {
        "numericTag": HNum(25),
        "stringTag": HStr("test string"),
        "refTag": HRef("thisIsTheRef"),
        "boolTag": HBool.FALSE
      };

      HDict record1 = HDict(recordTags);
      HDict record2 = HDict(recordTags);
      HDict record3 = HDict(recordTags);
      HDict emptyRecord1 = HDict({});
      HDict emptyRecord2 = HDict({});
      HDict meta = HDict({});

      List<HDict> emptyInTheMiddle = [
        record1,
        emptyRecord1,
        record2,
        emptyRecord2,
        record3
      ];

      List<HDict> emptyAtStart = [
        emptyRecord1,
        emptyRecord2,
        record1,
        record2,
        record3
      ];

      List<HDict> emptyAtTheEnd = [
        record1,
        record2,
        record3,
        emptyRecord1,
        emptyRecord2
      ];


      List<List<HDict>> gridsData = [emptyInTheMiddle, emptyAtStart, emptyAtTheEnd];

      gridsData.forEach((List<HDict> gridData) {
        HGrid testGrid = HGridBuilder.dictsToGrid(meta, gridData);
        StringBuffer stringBuffer = StringBuffer();
        HTrioWriter writer = HTrioWriter(stringBuffer);
        writer.writeGrid(testGrid);

        String result = stringBuffer.toString();

        int delimitersCount = result.split("\n-").length - 1;
        bool isFirstDelimiter = result.codeUnitAt(0) == MINUS_CHAR;
        if (isFirstDelimiter) delimitersCount++;

        expect(delimitersCount, 4, reason: "Must be 4 delimiters for grid with 5 records.");
      });
    });
  });
}